-- prepared by Steve Stedman
-- http://SteveStedman.com
-- twitter:  @SqlEmt
-- TSQL JOIN Types


USE Master;
GO

IF EXISTS(SELECT name 
            FROM sys.databases 
		   WHERE name = 'QueryTraining')
BEGIN
	ALTER DATABASE [QueryTraining] 
	  SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [QueryTraining];
END
CREATE DATABASE [QueryTraining];
GO

USE [QueryTraining];
GO

CREATE TABLE Table1
(
	id   INTEGER IDENTITY NOT NULL,
	Name VARCHAR(100) NOT NULL,
    fk   INTEGER NULL,
    fk_table3 INTEGER NULL,
PRIMARY KEY (id)
) ;

CREATE TABLE Table2
(
	id   INTEGER NOT NULL,
	FavoriteColor VARCHAR(100) NOT NULL
) ;
-- notice... Table2 is more real world, kinda slopy with no PRIMARY KEY

CREATE TABLE Table3
(
	id   INTEGER NOT NULL,
	dataValue VARCHAR(100) NOT NULL
) ;


INSERT INTO Table1 (Name, fk, fk_table3) VALUES
    ('Steve', 1, NULL),('Aaron', 3, NULL),('Mary', 2, NULL),
    ('Fred', 1, NULL),('Anne', 5, NULL),('Beth', 8, 1),
    ('Johnny', NULL, 1), ('Karen', NULL, 2);

INSERT INTO Table2 (id, FavoriteColor) VALUES
    (1, 'red'),(2, 'green'),(3, 'blue'),
    (4, 'pink'),(5, 'purple'),(6, 'mauve'),
    (7, 'orange'),(8, 'yellow'),(1, 'indigo');
-- note that red and indigo both have id's of 1

INSERT INTO Table3 (id, DataValue) VALUES
    (1, 'Pizza'),(2, 'Burger'),(3, 'Sushi');

-- SELECT FROM TWO TABLES
SELECT *
  FROM Table1;

SELECT * 
  FROM Table2;


SET STATISTICS IO ON;
-- INNER JOIN
SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk = t2.id
ORDER BY t1.id;

-- WHAT ABOUT THIS???
SELECT *
  FROM Table1 t1
  JOIN Table2 t2
    ON t1.fk = t2.id;

    
    
-- LEFT OUTER JOIN
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
ORDER BY t1.id;
    

-- RIGHT OUTER JOIN
SELECT *
  FROM Table1 t1
 RIGHT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
ORDER BY t1.id;
    

-- multiple OUTER JOINS
SELECT *
  FROM Table1 t1
 RIGHT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
 LEFT OUTER JOIN Table3 t3
   ON t1.fk_table3 = t3.id
ORDER BY t1.id;



-- WHAT ABOUT THIS???
SELECT *
  FROM Table1 t1
 OUTER JOIN Table2 t2
    ON t1.fk = t2.id;




    
-- SEMI JOIN 
SELECT *
  FROM Table1 t1
 WHERE EXISTS (SELECT 1
                 FROM Table2 t2
                WHERE t1.fk = t2.id
              ); 
-- Show me all the rows in Table1 that do match Table2
--  but only return the columns from table 1.
               
-- ANTI SEMI JOIN
SELECT *
  FROM Table1 t1
 WHERE NOT EXISTS (SELECT 1
                     FROM Table2 t2
		 	        WHERE t1.fk = t2.id
			       ); 
-- show me all the rows in Table1 that don't match Table2                   
--  but only return the columns from table 1.
--  could not not just say where t1.fk IS NOT NULL?
--  the anti-semi-join confirms that the match exists



-- LEFT OUTER JOIN with Exclusion
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
 WHERE t2.id IS NULL;
-- give me everyone from table 1 who doesn't match table 1


  
 
-- RIGHT OUTER JOIN with Exclusion
SELECT *
  FROM Table1 t1
  RIGHT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
 WHERE t1.fk IS NULL;
-- show me all rows from Table2 that don't have a match on Table1 


 -- WHAT ABOUT THIS???
SELECT *
  FROM Table1 t1
 OUTER JOIN Table2 t2
    ON t1.fk = t2.id;


-- FULL OUTER JOIN 
SELECT *
  FROM Table1 t1
  FULL OUTER JOIN Table2 t2
    ON t1.fk = t2.id;


-- FULL OUTER JOIN with exclusions
SELECT *
  FROM Table1 t1
  FULL OUTER JOIN Table2 t2
    ON t1.fk = t2.id
 WHERE t1.fk IS NULL
    OR t2.id IS NULL;
-- Show me everyone from Table1 and Table2 that don't match


-- CROSS JOIN
SELECT *
  FROM Table1 t1
  CROSS JOIN Table2 t2;
-- For every row in Table1 match it up with every row from Table2
-- no join predicate.


-- LATERAL JOIN - CROSS APPLY
SELECT *
  FROM Table1 t1;




SELECT *
  FROM Table1 t1
 CROSS APPLY
       (SELECT *
          FROM Table2 t2
         WHERE t2.id = t1.fk
       ) AS t; 
-- compared to INNER JOIN
SET STATISTICS IO ON;
SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk = t2.id;

-- but you can't accomplish the correlated function call with an inner join
GO
-- lets look at CROSS APPLY with a function
--DROP FUNCTION [dbo].[getFavoriteColorMulti] ;
GO
CREATE FUNCTION [dbo].[getFavoriteColorMulti] (@idFilter AS INTEGER)
RETURNS @favoriteColor TABLE	
(
	id   INTEGER NOT NULL,
	FavoriteColor VARCHAR(100) NOT NULL
)
AS 
BEGIN
	INSERT INTO @favoriteColor (id, FavoriteColor)
	SELECT id, FavoriteColor
	  FROM Table2
	 WHERE id = @idFilter;

	 RETURN;
END

SELECT * 
  FROM [dbo].[getFavoriteColorMulti] (2);


SELECT *
  FROM Table1 t1
 CROSS APPLY
       (SELECT *
          FROM Table2 t2
         WHERE t2.id = t1.fk
       ) AS t; 

SELECT *
  FROM Table1 t1
 CROSS APPLY [dbo].[getFavoriteColorMulti](t1.fk) AS t; 

GO
CREATE FUNCTION [dbo].[getFavoriteColorSingle] (@idFilter AS INTEGER)
RETURNS TABLE 
AS 
RETURN
	SELECT id, FavoriteColor
	  FROM Table2
	 WHERE id = @idFilter;
GO

SELECT *
  FROM Table1 t1
 CROSS APPLY [dbo].[getFavoriteColorSingle](t1.fk) AS t; 

SET STATISTICS IO OFF
-- turn off execution plan with ctrl+m
GO
INSERT INTO Table1 (Name, fk, fk_table3) VALUES
    (cast(NEWID() AS varchar(100)), NULL, NULL);
GO 10000
SELECT * FROM Table1;
SET STATISTICS IO ON
-- turn on execution plan with ctrl+m





-- CROSS APPLY
SELECT *
  FROM Table1 t1
 CROSS APPLY [dbo].[getFavoriteColorSingle](t1.fk) AS t; 

SELECT *
  FROM Table1 t1
 CROSS APPLY [dbo].[getFavoriteColorMulti](t1.fk) AS t; 

SELECT *
  FROM Table1 t1
 CROSS APPLY
       (SELECT *
          FROM Table2 t2
         WHERE t2.id = t1.fk
       ) AS t; 

 
-- type typical use of CROSS APPLY
SELECT qs.*
  FROM sys.dm_exec_query_stats AS qs
 CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle)


-- OUTER APPLY

SELECT *
  FROM Table1 t1
 OUTER APPLY [dbo].[getFavoriteColorSingle](t1.fk) AS t; 

SELECT *
  FROM Table1 t1
 OUTER APPLY [dbo].[getFavoriteColorMulti](t1.fk) AS t; 

SELECT *
  FROM Table1 t1
 OUTER APPLY
       (SELECT *
          FROM Table2 t2
         WHERE t2.id = t1.fk
       ) AS t; 



-- NON-EQUI INNER JOIN
SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk >= t2.id;
    
    
-- NON-EQUI LEFT OUTER JOIN
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk >= t2.id;
    

-- NON-EQUI RIGHT OUTER JOIN
SELECT *
  FROM Table1 t1
 RIGHT OUTER JOIN Table2 t2
    ON t1.fk < t2.id;




-- TWO INNER JOINS

SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk = t2.id
 INNER JOIN Table3 t3
    ON t1.fk_table3 = t3.id;
    
    
-- Two LEFT OUTER JOINs    
SELECT *
  FROM Table1 t1
  LEFT OUTER JOIN Table2 t2
    ON t1.fk = t2.id
  LEFT OUTER JOIN Table3 t3
    ON t1.fk_table3 = t3.id;
    

-- INNER JOIN with a LEFT OUTER JOIN
SELECT *
  FROM Table1 t1
 INNER JOIN Table2 t2
    ON t1.fk = t2.id
  LEFT OUTER JOIN Table3 t3
    ON t1.fk_table3 = t3.id; 

-- TWO FULL OUTER JOINS
SELECT *
  FROM Table1 t1
  FULL OUTER JOIN Table2 t2
    ON t1.fk = t2.id
  FULL OUTER JOIN Table3 t3
    ON t1.fk_table3 = t3.id; 






-- EXCEPT, INTERSECT AND UNION On the poster, but not in the presenation





SELECT * FROM Table1;
SELECT * FROM Table2;
-- EXCEPT
SELECT fk as id
  FROM Table1
EXCEPT
SELECT ID 
  FROM Table2;

SELECT ID 
  FROM Table2
EXCEPT
SELECT fk as id
  FROM Table1;




SELECT * FROM Table1;
SELECT * FROM Table2;

-- INTERSECT
SELECT fk as id
  FROM Table1
INTERSECT 
SELECT ID 
  FROM Table2;



SELECT * FROM Table1;
SELECT * FROM Table2;

-- UNION
SELECT fk as id
  FROM Table1
UNION
SELECT ID 
  FROM Table2;

-- includes duplicates  
SELECT fk as id
  FROM Table1
UNION ALL
SELECT ID 
  FROM Table2;

